
import numpy as np
"""
File name: world_environment
Date created: 19/02/2019
Feature: # Creation of Environment for Reinforcement Learning
"""

__author__ = "Alexander Kell"
__copyright__ = "Copyright 2018, Alexander Kell"
__license__ = "MIT"
__email__ = "alexander@kell.es"
